module Cursor(
input	[9:0]	iR,
input	[9:0]	iG,
input	[9:0]	iB,
input iLeft,
input	iUp,
input	iDown,
input iRight,
output [9:0] oR,
output [9:0] oG,
output [9:0] oB,
input [10:0] iVGA_X,
input	[10:0] iVGA_Y,
output [10:0] oVGA_X,
output [10:0] oVGA_Y,
input	iRST,
input iCLK,
input slow,
input iCursorB,
input [1:0] ichangeC,
output [10:0] oCY,
output reg [10:0] cur_pointer_x
);

wire [3:0] speed_x, speed_y;
reg [10:0] cur_X, cur_Y;
reg arrow;
reg [20:0] debounce;
reg debounce1;
reg [10:0] cur_pointer_y;

assign oR = (arrow ? (iCursorB ? iG : 10'h3FF) : iR);
assign oG = (arrow ? (iCursorB ? iR : 10'd0) : iG);
assign oB = (arrow ? (iCursorB ? iB : 10'd0) : iB);

parameter fnote_up_x=11'd45, hnote_up_x=64, qnote_up_x=11'd83, enote_up_x=11'd102, snote_up_x=11'd122, tnote_up_x=11'd144, sfnote_up_x=11'd165;
parameter fnote_up_y=11'd76, hnote_up_y=76, qnote_up_y=11'd76, enote_up_y=11'd76, snote_up_y=11'd76, tnote_up_y=11'd76, sfnote_up_y=11'd76;
parameter frest_up_x=11'd320, hrest_up_x=11'd339, qrest_up_x=11'd358, erest_up_x=11'd378, srest_up_x=11'd397, trest_up_x=11'd415, sfrest_up_x=11'd435;
parameter frest_up_y=11'd76, hrest_up_y=11'd76, qrest_up_y=11'd76, erest_up_y=11'd76, srest_up_y=11'd76, trest_up_y=11'd76, sfrest_up_y=11'd76;

assign oCY = cur_Y;


assign speed_x = (slow ? 4'd1 : 4'hF );
assign speed_y = (slow ? 4'd1 : 4'h3 );
assign oVGA_X = (arrow ? cur_pointer_x + iVGA_X - cur_X : iVGA_X);
assign oVGA_Y = (arrow ? cur_pointer_y + iVGA_Y - cur_Y : iVGA_Y);


always @(posedge iCLK)
begin
 if(iRST) begin
		cur_X <= 11'd320;
		cur_Y <= 11'd240;
		arrow <= 1'b0;
		cur_pointer_x <= qnote_up_x;
		cur_pointer_y <= qnote_up_y;
		debounce <= 0;
 end
 else begin
		if ( ((iVGA_X > cur_X - 12) && (iVGA_X < cur_X + 12))) begin
				if (((iVGA_Y > cur_Y - 34) && (iVGA_Y < cur_Y + 34))) 
					arrow <= 1'b1;	
				else 
					arrow <= 1'b0;
		end
		else 
					arrow <= 1'b0;
		if ( iLeft ) begin
			if (debounce == 0) 
				cur_X <= cur_X - speed_x;
			debounce <= debounce + 1;
		end
		else if ( iRight  ) begin
			if (debounce == 0) 
				cur_X <= cur_X + speed_x;
			debounce <= debounce + 1;
			
		end
		else if ( iUp ) begin
			if (debounce == 0) 
				cur_Y <= cur_Y - speed_y;
			debounce <= debounce + 1;
				
		end
		else if ( iDown ) begin
			if (debounce == 0) 
				cur_Y <= cur_Y + speed_y;
			debounce <= debounce + 1;
				
		end
		else 
			debounce <= 0;
		
		if (ichangeC[0]) begin
			if (debounce1 == 0) begin 		
			 case (cur_pointer_x)	
				fnote_up_x: 	begin 	cur_pointer_x <= hnote_up_x;
												cur_pointer_y <= hnote_up_y;   	end
				hnote_up_x: 	begin 	cur_pointer_x <= qnote_up_x;
												cur_pointer_y <= qnote_up_y;   	end
				qnote_up_x: 	begin 	cur_pointer_x <= enote_up_x;
												cur_pointer_y <= enote_up_y;   	end
				enote_up_x: 	begin 	cur_pointer_x <= snote_up_x;
												cur_pointer_y <= snote_up_y;   	end
				snote_up_x: 	begin 	cur_pointer_x <= tnote_up_x;
												cur_pointer_y <= tnote_up_y;   	end
				tnote_up_x: 	begin 	cur_pointer_x <= sfnote_up_x;
												cur_pointer_y <= sfnote_up_y;   	end
				sfnote_up_x: 	begin 	cur_pointer_x <= frest_up_x;
												cur_pointer_y <= frest_up_y;   	end											
				frest_up_x: 	begin 	cur_pointer_x <= hrest_up_x;
												cur_pointer_y <= hrest_up_y;   	end
				hrest_up_x: 	begin 	cur_pointer_x <= qrest_up_x;
												cur_pointer_y <= qrest_up_y;   	end
				qrest_up_x: 	begin 	cur_pointer_x <= erest_up_x;
												cur_pointer_y <= erest_up_y;   	end
				erest_up_x: 	begin 	cur_pointer_x <= srest_up_x;
												cur_pointer_y <= srest_up_y;   	end
				srest_up_x: 	begin 	cur_pointer_x <= trest_up_x;
												cur_pointer_y <= trest_up_y;   	end
				trest_up_x: 	begin 	cur_pointer_x <= sfrest_up_x;
												cur_pointer_y <= sfrest_up_y;   	end
				sfrest_up_x: 	begin 	cur_pointer_x <= fnote_up_x;
												cur_pointer_y <= fnote_up_y;   	end									
												
			 endcase
			end
			debounce1 <= 1;
		end
		else if(ichangeC[1]) begin
			if (debounce1 == 0) begin 
			 case (cur_pointer_x)	
				fnote_up_x: 	begin 	cur_pointer_x <= sfrest_up_x;
												cur_pointer_y <= sfrest_up_y;   	end
				hnote_up_x: 	begin 	cur_pointer_x <= fnote_up_x;
												cur_pointer_y <= fnote_up_y;   	end
				qnote_up_x: 	begin 	cur_pointer_x <= hnote_up_x;
												cur_pointer_y <= hnote_up_y;   	end
				enote_up_x: 	begin 	cur_pointer_x <= qnote_up_x;
												cur_pointer_y <= qnote_up_y;   	end
				snote_up_x: 	begin 	cur_pointer_x <= enote_up_x;
												cur_pointer_y <= enote_up_y;   	end
				tnote_up_x: 	begin 	cur_pointer_x <= snote_up_x;
												cur_pointer_y <= snote_up_y;   	end
				sfnote_up_x: 	begin 	cur_pointer_x <= tnote_up_x;
												cur_pointer_y <= tnote_up_y;   	end
												
				frest_up_x: 	begin 	cur_pointer_x <= sfnote_up_x;
												cur_pointer_y <= sfnote_up_y;   	end
				hrest_up_x: 	begin 	cur_pointer_x <= frest_up_x;
												cur_pointer_y <= frest_up_y;   	end
				qrest_up_x: 	begin 	cur_pointer_x <= hrest_up_x;
												cur_pointer_y <= hrest_up_y;   	end
				erest_up_x: 	begin 	cur_pointer_x <= qrest_up_x;
												cur_pointer_y <= qrest_up_y;   	end
				srest_up_x: 	begin 	cur_pointer_x <= erest_up_x;
												cur_pointer_y <= erest_up_y;   	end
				trest_up_x: 	begin 	cur_pointer_x <= srest_up_x;
												cur_pointer_y <= srest_up_y;   	end
				sfrest_up_x: 	begin 	cur_pointer_x <= trest_up_x;
												cur_pointer_y <= trest_up_y;   	end
			 endcase
			end
			debounce1 <= 1;
		end
		else begin
			debounce1 <= 0;
		end
 end
end


endmodule
