`include "scan_events.h"

// Module that receives user input in the form of switches, buttons,
// and keyboard scan events, and outputs user-controlled properties
// like cursor size, color, and position, faucet sizes, and reset,
// pause, and troll commands.
module Input_Controller (
    input             iCLK,              // System clock signal
    input             iRESET,            // System reset signal (power-on reset)
    input             iEventReady,       // Keyboard event ready
    input      [7:0]  iEventType,        // Type of keyboard event that is ready
	 output	reg		 oRight_Arrow,
	 output	reg		 oLeft_Arrow,
    output	reg		 oUp_Arrow,
	 output	reg		 oDown_Arrow,
	 output  reg		 oDraw,
	 output  reg		 oChangeL,
	 output  reg 		 oChangeR,
	 output  reg       oSlow,
	 output  reg		 oReset,
	 output	reg		 oPlay
    
);
    
    // Detect keyboard input during all cycles, but only act on it
    // once per frame at the iNewFrame signal.
    reg       HOLDING_DRAW;
    reg       HOLDING_LEFT_ARROW;
    reg       HOLDING_RIGHT_ARROW;
    reg       HOLDING_UP_ARROW;
    reg       HOLDING_DOWN_ARROW;
	 reg		  HOLDING_COMMA;
	 reg		  HOLDING_PERIOD;
	 reg		  HOLDING_CONTROL;
	 reg		  HOLDING_ESC;
	 reg		  HOLDING_ENTER;
   
    // Detect key holds.
    always @(posedge iCLK) begin
        if(iRESET) begin
            HOLDING_DRAW         <= 1'b0;
            HOLDING_LEFT_ARROW   <= 1'b0;
            HOLDING_RIGHT_ARROW  <= 1'b0;
            HOLDING_UP_ARROW     <= 1'b0;
            HOLDING_DOWN_ARROW   <= 1'b0;
				HOLDING_COMMA			<= 1'b0;
				HOLDING_PERIOD			<= 1'b0;
				HOLDING_CONTROL		<= 1'b0;
				HOLDING_ESC				<= 1'b0;
				HOLDING_ENTER			<= 1'b0;
				
        end
        else begin
            if(iEventType == `SC_MAKE_SPACE)             HOLDING_DRAW        <= 1'b1;
            else if(iEventType == `SC_BREAK_SPACE)       HOLDING_DRAW        <= 1'b0;
            if(iEventType == `SC_MAKE_LEFT_ARROW)        HOLDING_LEFT_ARROW  <= 1'b1;
            else if(iEventType == `SC_BREAK_LEFT_ARROW)  HOLDING_LEFT_ARROW  <= 1'b0;
            if(iEventType == `SC_MAKE_RIGHT_ARROW)       HOLDING_RIGHT_ARROW <= 1'b1;
            else if(iEventType == `SC_BREAK_RIGHT_ARROW) HOLDING_RIGHT_ARROW <= 1'b0;
            if(iEventType == `SC_MAKE_UP_ARROW)          HOLDING_UP_ARROW    <= 1'b1;
            else if(iEventType == `SC_BREAK_UP_ARROW)    HOLDING_UP_ARROW    <= 1'b0;
            if(iEventType == `SC_MAKE_DOWN_ARROW)        HOLDING_DOWN_ARROW  <= 1'b1;
            else if(iEventType == `SC_BREAK_DOWN_ARROW)  HOLDING_DOWN_ARROW  <= 1'b0;
				if(iEventType == `SC_MAKE_COMMA)        		HOLDING_COMMA  	  <= 1'b1;
            else if(iEventType == `SC_BREAK_COMMA)  		HOLDING_COMMA  	  <= 1'b0;
				if(iEventType == `SC_MAKE_PERIOD)       		HOLDING_PERIOD  	  <= 1'b1;
            else if(iEventType == `SC_BREAK_PERIOD)  		HOLDING_PERIOD  	  <= 1'b0;
				if(iEventType == `SC_MAKE_RT_CTRL)      	   HOLDING_CONTROL  	  <= 1'b1;
            else if(iEventType == `SC_BREAK_RT_CTRL)	   HOLDING_CONTROL  	  <= 1'b0;
				if(iEventType == `SC_MAKE_ESC)      	      HOLDING_ESC     	  <= 1'b1;
            else if(iEventType == `SC_BREAK_ESC)	      HOLDING_ESC    	  <= 1'b0;
				if(iEventType == `SC_MAKE_ENTER)      	      HOLDING_ENTER    	  <= 1'b1;
            else if(iEventType == `SC_BREAK_ENTER)	      HOLDING_ENTER    	  <= 1'b0;
				
        end
    end
    


    

    // Manage cursor position.
    always @(posedge iCLK) begin
                 // When receiving input from the keyboard, use the
                // held arrow keys to move the cursor.
                if(HOLDING_LEFT_ARROW & ~HOLDING_RIGHT_ARROW)
                    oLeft_Arrow <= 1'b1;
  				    else
						  oLeft_Arrow <= 1'b0;
						  
                if(HOLDING_RIGHT_ARROW & ~HOLDING_LEFT_ARROW) 
                    oRight_Arrow <= 1'b1;
                else
						  oRight_Arrow <= 1'b0;
						  
                if(HOLDING_UP_ARROW & ~HOLDING_DOWN_ARROW) 
						  oUp_Arrow <= 1'b1;
                else
						  oUp_Arrow <= 1'b0;
						  
                if(HOLDING_DOWN_ARROW & ~HOLDING_UP_ARROW) 
						  oDown_Arrow <= 1'b1;
                else
						  oDown_Arrow <= 1'b0;
						  
					 if(HOLDING_DRAW)
						  oDraw <= 1'b1;
					 else
						  oDraw <= 1'b0;
						  
					 if(HOLDING_COMMA)
						  oChangeL <= 1'b1;
					 else
						  oChangeL <= 1'b0;
					 
					 if(HOLDING_PERIOD)
						  oChangeR <= 1'b1;
					 else
						  oChangeR <= 1'b0;
						  
					 if(HOLDING_CONTROL)
						  oSlow <= 1'b1;
					 else
						  oSlow <= 1'b0;
						  
					 if(HOLDING_ESC)
						  oReset <= 1'b1;
					 else
						  oReset <= 1'b0;
						  
					 if(HOLDING_ENTER)
						  oPlay <= 1'b1;
					 else
						  oPlay <= 1'b0;
    end
    
endmodule
